#!/usr/bin/env python3
# -*- coding: utf-8 -*-


__author__ = 'luigelo@ldvloper.com'

"""
    Modules
"""
import os
from functools import wraps
from inspect import iscoroutinefunction, getfile
from typing import Any, Callable, TypeVar, cast

"""
    Result4Utility Modules
"""
from result4utility.result import Result
from result4utility.result import ResultOperationType as resType


class Decorest:

    Fn = TypeVar('Fn', bound=Callable[..., Any])

    def __init__(self):
        pass

    def try_log(self, func: Fn):
        if iscoroutinefunction(func):
            @wraps(func)
            async def wrapper(*args, **kwargs):
                try:
                    return await func(*args, **kwargs)
                except Exception as exc:
                    return self.raise_exception(func, exc)
            return cast(self.Fn, wrapper)
        else:
            @wraps(func)
            def wrapper(*args, **kwargs):
                try:
                    return func(*args, **kwargs)
                except Exception as exc:
                    return self.raise_exception(func, exc)
            return wrapper

    def raise_exception(self, func, exc, result: Result = None) -> Result:
        if not result:
            result = Result()
        result.add_result(message=f'Error occurred in {func.__name__}: {str(exc)} in {get_filename(func)}',
                          result_type=resType.EXCEPTION)
        return result


def get_filename(func):
    if getfile(func):
        return os.path.split(getfile(func))[-1]
    return ''



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__author__ = 'luigelo@ldvloper'

"""
    Modules
"""
from copy import deepcopy


class Tools(object):
    """
        Tool Class - For common functionalities
    """

    def __init__(self):
        """
            Basic constructor
        """
        self.__dictionary: dict = dict()
        pass

    def remove_property(self, target: dict, props: []) -> dict:
        """
            Method used for remove properties from a dictionary
            :rtype: object
            :param target: dict. Dictionary to apply changes
            :param props: []. Properties of dictionary that we will remove inside.
        """
        try:
            for p in props:
                del target[p]
            self.__dictionary.update(target)
        except Exception:
            pass
        return self.__dictionary

    def dictionary_from_object(self, model: object) -> dict:
        """
            Method used for convert model into dictionary
            :rtype: dict
            :param model: object. Object over that we can do conversion to dictionary. Previously we do an copy and convert this copy.
        """
        try:
            bck_model = deepcopy(model)
            self.__dictionary = bck_model.__dict__
        except Exception:
            pass
        return self.__dictionary



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__author__ = 'luigelo@ldvloper.com'

"""
    Modules
"""
from enum import Enum


class ResultOperation(object):
    def __init__(self, message, result_type):
        self.__message = message
        self.__result_type = result_type

    @property
    def message(self):
        return self.__message

    @property
    def result_type(self):
        return self.__result_type


class ResultOperationType(Enum):
    ERROR = 0
    INFO = 1
    WARNING = 2
    SUCCESS = 3
    EXCEPTION = 4
